%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the method for computing the Imbalance Ratio formulation presented in the paper Graph-Based Approaches for Over-sampling in the context of Ordinal Regression published in the IEEE Transactions on Knowledge and Data Engineering. 
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/GBOforOR
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [initial_irs, irs, n] = computeNumberOfPatternstoOversample(train)
% Compute the imbalance ratio for all of the classes in the dataset and the number of patterns to be synthetically created for each class
% Input variables: 
% *train: train structure containing the train patterns and the associated
%  targets
% Output variables
% *initial_irs: Vector of initial Imbalanced Ratio per class
% *irs: Vector of Imbalanced Ratio (IR) per class after the iterative proccess
% *n: Vector containing the number of patterns to create for each class (so that the imbalance ratio of this class is lower than 1.5)

nclases = numel(unique(train.targets));
salir=1;
n = zeros(1,nclases);

for i=1:nclases,
    initial_irs(i) = (sum(train.targets~=i)/sum(train.targets==i))/nclases;
end

while(salir~=nclases)
    salir = 0;
    for i=1:nclases,
        irs(i) = ((sum(train.targets~=i)+(sum(n)-n(i)))/(sum(train.targets==i)+n(i)))/nclases;
        if irs(i)>1.5,
            n(i) = ceil((sum(train.targets~=i)+(sum(n)-n(i)))/(1.5*nclases) - sum(train.targets==i));
        else
            salir = salir + 1;
        end
    end
end

end
