%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the ordinal graph-based approaches for over-sampling of the paper Graph-Based Approaches for Over-sampling in the context of Ordinal Regression published in the IEEE Transactions on Knowledge and Data Engineering. 
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/GBOforOR
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

addpath tools/

% Example code for a toy ordinal dataset.
load toy-train
load toy-test

nOfClasses = numel(unique(train.targets));
% Whether to use the gamma probability function for the intra-class edges of the graph
probability_function = 1;
% Use of shortest paths
shortest_paths = 1;

% Compute the IR for each class and the number of patterns to create
[initial_irs, irs, n] = computeNumberOfPatternstoOversample(train);
n = ceil(n);
k = 5;

trainNEW = train;
for i=1:nOfClasses,
       if n(i)~=0,
	      if ~shortest_paths,
              		sample = OGONI(train,i,k,n(i));
	      else
		        if probability_function,
				      sample = OGOSP(train,i,k,n(i));
			else
				      sample = OGOISP(train,i,k,n(i));
			end
	      end
              
	      trainNEW.patterns = [trainNEW.patterns; sample];
              trainNEW.targets = [trainNEW.targets; (ones(1,n(i))*i)'];
       end
       train = trainNEW;
end            

